grammar myGrammar;

compilationUnit
    :   (extDeclaration | nonExtDeclaration | nonExtFunctionDeclaration | extFunctionDeclaration |
         unionDeclaration | .)*? EOF
    ;

extDeclaration
    :   Extern Type Identifier Identifier? Brackets?  Attribution?(',' Identifier Brackets? Attribution?)*?';'
    |   Extern Type '{' (unionDeclaration | nonExtDeclaration | .)*? '}' Identifier Brackets?';'
    ;

nonExtDeclaration
    :   Type Identifier Identifier? Brackets? Attribution? (',' Identifier Brackets? Attribution?)*?';'
    |   Type '{' (unionDeclaration | nonExtDeclaration | .)*? '}' Identifier Brackets?';'
    ;

extFunctionDeclaration
    :   Extern Type Identifier '(' arguments? ')' '{' (braceDeclarations | nonExtDeclaration | .)*? '}'
    |   Extern Type Identifier '(' arguments? ')' ';'
    ;

nonExtFunctionDeclaration
    :   Type Identifier '(' arguments? ')' '{' (braceDeclarations | nonExtDeclaration | .)*? '}'
    |   Type Identifier '(' arguments? ')' ';'
    ;

braceDeclarations
    :   ifStatement
    |   whileStatement
    |   forStatement
    |   doWhile
    ;

ifStatement
    :   If '(' .*? ')' (('{' (braceDeclarations | nonExtDeclaration | .)*? '}') | ((nonExtDeclaration | .*?)';'))
    |   Else (ifStatement | ('{' (braceDeclarations | nonExtDeclaration | .)*? '}') | ((nonExtDeclaration | .*?)';'))
    ;

whileStatement
    :   While '(' .*? ')' ('{' (braceDeclarations | nonExtDeclaration | .)*? '}')?
    ;

forStatement
    :   For '(' .*? ')' ('{' (braceDeclarations | nonExtDeclaration | .)*? '}')?
    ;

doWhile
    :   Do '{' (braceDeclarations | nonExtDeclaration | .)*? '}'
    ;

arguments
    : argument (',' argument)*
    ;

argument
    :   Type Identifier Identifier?
    |   Type
    ;

unionDeclaration
    :   Extern? Union '{' (nonExtDeclaration | Identifier | .)*? '}' Identifier Brackets?';'
    ;

Type
    :   'int'
    |   'struct'
    |   'float'
    |   'void'
    |   'char'
    |   'short'
    |   'long'
    |   'double'
    |   'signed'
    |   'unsigned'
    ;

Union
    :   'union'
    ;

Extern
    :   'extern'
    ;

Static
    :   'static'
    ;

If
    :   'if'
    ;

Else
    :   'else'
    ;

While
    :   'while'
    ;

For
    :   'for'
    ;

Do
    :   'do'
    ;

Identifier
    :   [a-zA-Z_*]+ [0-9]*
    ;

Ws
    :   [ \n\r\t]+ -> skip
    ;

Unused
    :   [_\!\?\|\~\&\@\:\,\#\{\}\%\[\]\<\>\/\\\^\$\*\=\+\'\"\`\.0-9\-]+ -> skip
    ;

Attribution
    :   [=] [a-zA-Z _\@\`\!\?\|\$\^\~\&\:\(\)\#\{\}\%\[\]\<\>\/\\\*\=\+\'\"\.0-9\-]+
    ;

Brackets
    :   [\[] [a-zA-Z \+\*\-\_\/0-9]* [\]]
    ;