/**
 * Copyright 2017 Jose Nelson Amaral
 * Copyright 2017 Joao Hoffman
 * Copyright 2017 Morgan Redshaw
 * Copyright 2017 Erick Ochoa Lopez
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

grammar myGrammar;

compilationUnit
    :   (extDeclaration | nonExtDeclaration | nonExtFunctionDeclaration | extFunctionDeclaration |
         unionDeclaration | .)*? EOF
    ;

extDeclaration
    :   Extern Type Identifier Identifier? Brackets?  Attribution?(',' Identifier Brackets? Attribution?)*?';'
    |   Extern Type '{' (unionDeclaration | nonExtDeclaration | .)*? '}' Identifier Brackets?';'
    ;

nonExtDeclaration
    :   Type Identifier Identifier? Brackets? Attribution? (',' Identifier Brackets? Attribution?)*?';'
    |   Type '{' (unionDeclaration | nonExtDeclaration | .)*? '}' Identifier Brackets?';'
    ;

extFunctionDeclaration
    :   Extern Type Identifier '(' arguments? ')' '{' (braceDeclarations | nonExtDeclaration | .)*? '}'
    |   Extern Type Identifier '(' arguments? ')' ';'
    ;

nonExtFunctionDeclaration
    :   Type Identifier '(' arguments? ')' '{' (braceDeclarations | nonExtDeclaration | .)*? '}'
    |   Type Identifier '(' arguments? ')' ';'
    ;

braceDeclarations
    :   ifStatement
    |   whileStatement
    |   forStatement
    |   doWhile
    ;

ifStatement
    :   If '(' .*? ')' (('{' (braceDeclarations | nonExtDeclaration | .)*? '}') | ((nonExtDeclaration | .*?)';'))
    |   Else (ifStatement | ('{' (braceDeclarations | nonExtDeclaration | .)*? '}') | ((nonExtDeclaration | .*?)';'))
    ;

whileStatement
    :   While '(' .*? ')' ('{' (braceDeclarations | nonExtDeclaration | .)*? '}')?
    ;

forStatement
    :   For '(' .*? ')' ('{' (braceDeclarations | nonExtDeclaration | .)*? '}')?
    ;

doWhile
    :   Do '{' (braceDeclarations | nonExtDeclaration | .)*? '}'
    ;

arguments
    : argument (',' argument)*
    ;

argument
    :   Type Identifier Identifier?
    |   Type
    ;

unionDeclaration
    :   Extern? Union '{' (nonExtDeclaration | Identifier | .)*? '}' Identifier Brackets?';'
    ;

Type
    :   'int'
    |   'struct'
    |   'float'
    |   'void'
    |   'char'
    |   'short'
    |   'long'
    |   'double'
    |   'signed'
    |   'unsigned'
    ;

Union
    :   'union'
    ;

Extern
    :   'extern'
    ;

Static
    :   'static'
    ;

If
    :   'if'
    ;

Else
    :   'else'
    ;

While
    :   'while'
    ;

For
    :   'for'
    ;

Do
    :   'do'
    ;

Identifier
    :   [a-zA-Z_*]+ [0-9]*
    ;

Ws
    :   [ \n\r\t]+ -> skip
    ;

Unused
    :   [_\!\?\|\~\&\@\:\,\#\{\}\%\[\]\<\>\/\\\^\$\*\=\+\'\"\`\.0-9\-]+ -> skip
    ;

Attribution
    :   [=] [a-zA-Z _\@\`\!\?\|\$\^\~\&\:\(\)\#\{\}\%\[\]\<\>\/\\\*\=\+\'\"\.0-9\-]+
    ;

Brackets
    :   [\[] [a-zA-Z \+\*\-\_\/0-9]* [\]]
    ;
