/**
 * Copyright 2017 Jose Nelson Amaral
 * Copyright 2017 Joao Hoffman
 * Copyright 2017 Morgan Redshaw
 * Copyright 2017 Erick Ochoa Lopez
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Created by stark on 20/05/16.
 */

import java.io.*;

import org.antlr.v4.runtime.*;
import org.antlr.v4.runtime.tree.ParseTree;
import org.antlr.v4.runtime.tree.ParseTreeWalker;

public class nameMangling {

    public static void main (String [] args){

        if (args.length == 0){
            System.out.println("Error: input file expected.");
        }
        else if (args.length > 1){
            System.out.println("Error: number of inputs exceeds the maximum.");
        }
        else{
            // Gets the lexer
            C_manglingLexer lexer = null;
            try {
                lexer = new C_manglingLexer(new ANTLRFileStream(args[0]));
            } catch (IOException e) {
                e.printStackTrace();
            }

            // Gets the list of tokens
            CommonTokenStream tokens = new CommonTokenStream(lexer);

            // Provides the tokens to the parser
            C_manglingParser parser = new C_manglingParser(tokens);

            // Specifies the entry point
            ParserRuleContext tree = parser.compilationUnit();

            // Walks in and use the listener assigned to define symbols
            ParseTreeWalker walker = new ParseTreeWalker();
            defineListener define = new defineListener();
            walker.walk(define, tree);

            // Walks in and use the listener assigned to resolve symbols
            resolveListener resolve = new resolveListener(define.getMap());
            walker.walk(resolve, tree);

            // Prints not empty scopes and their symbols
            //define.printScopes();

            // Prints the final, mangled code
            resolve.printMangledCode(tokens);
        }
    }
}
